<?php

declare(strict_types=1);

namespace Drupal\entity_sync_group\Plugin\EntitySync\FieldTransformer;

// Drupal modules.
use Drupal\entity_sync\FieldTransformer\PluginBase;
// Drupal core.
use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
// Third-party libraries.
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Validator\Validator\ValidatorInterface;

/**
 * Transformer that converts an entity to an array containing its parent groups.
 *
 * Supported configuration properties:
 * - plugin_id: (string, optional, defaults to `NULL`) The ID of a relation
 *   plugin to limit the results to. If `NULL` the groups for relationships of
 *   all types will be returned.
 * - key_by: (string, optional, defaults to `relationship_id`) Determines how to
 *   index the results. Supported values are:
 *   - relationship_id: Use the ID of the relationship as the key for each item
 *     in the transformed value array. Note that some relationship types allow
 *     an entity to have multiple relationships to the same parent group. In
 *     that case, the transformed value array will contain the same group more
 *     than once keyed by different relationship IDs.
 *
 * Currently, only exports are supported. Support for imports should be added
 * here as well.
 *
 * @EntitySyncFieldTransformer(
 *   id = "group_entity_parent_group"
 * )
 * phpcs:disable
 * @I Support indexing by group ID
 *    type     : improvement
 *    priority : normal
 *    labels   : field-transformer
 * phpcs:enable
 */
class EntityParentGroup extends PluginBase {

  /**
   * Constructs a new EntityParentGroup object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Symfony\Component\Validator\Validator\ValidatorInterface $constraint_validator
   *   The constraint validator.
   * @param \Drupal\Component\Plugin\PluginManagerInterface $constraint_manager
   *   The constraint plugin manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    ValidatorInterface $constraint_validator,
    PluginManagerInterface $constraint_manager,
    protected EntityTypeManagerInterface $entityTypeManager,
  ) {
    parent::__construct(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $constraint_validator,
      $constraint_manager
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    ValidatorInterface $constraint_validator,
    array $configuration,
    $plugin_id,
    $plugin_definition,
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $constraint_validator,
      $container->get('plugin.manager.entity_sync_validation_constraint'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'plugin_id' => NULL,
      'key_by' => 'relationship_id',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  protected function transformExportedValue(
    $entity,
    ContentEntityInterface $local_entity,
    $remote_entity_id,
    array $field_info,
    array $context,
  ) {
    if ($entity === NULL) {
      return NULL;
    }

    if (!$entity instanceof EntityInterface) {
      throw new \RuntimeException(\sprintf(
        'The value must implement "%s".',
        EntityInterface::class,
      ));
    }

    $relationships = $this->entityTypeManager
      ->getStorage('group_relationship')
      ->loadByEntity($entity, $this->configuration['plugin_id']);

    return \array_map(fn ($r) => $r->getGroup(), $relationships);
  }

}
